/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.type.FileType;

import com.topcoder.util.config.Property;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * <p>
 * Defines a concrete implementation of <code>FileMatcher</code> interface to provide the functionality to match a
 * file against a list of MIME types. The MIME type should be the same as defined in Magic Numbers component. The
 * matching is done case-sensitively.
 * </p>
 *
 * <p>
 * The list of MIME types can be read from configuration under the property 'mimes'. This property must contain at
 * least one value. Empty strings are not allowed.
 * </p>
 *
 * <p>
 * This class is thead-safe, since it is immutable.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class MimeMatcher extends FileTypeMatcher {
    /** Represents the property name in configuration to read the list of MIME types. */
    public static final String MIMES_PROPERTY_KEY = "mimes";

    /** Represents the list of MIME types to be matched. */
    private List mimes;

    /**
     * <p>
     * Creates a new instance of <code>MimeMatcher</code> class. The list of MIME types contains only one given MIME
     * type.
     * </p>
     *
     * @param mime the MIME type to be matched.
     *
     * @throws NullPointerException if <code>mime</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>mime</code> is empty string.
     */
    public MimeMatcher(String mime) {
        Validator.validateNotNullOrEmpty(mime, "mime");

        mimes = new ArrayList();
        mimes.add(mime);
    }

    /**
     * <p>
     * Creates a new instance of <code>MimeMatcher</code> class. The list of MIME types is given. The list should
     * only contain non-empty strings, and cannot be empty list. Later modifications to the given list does not
     * affect the internal data.
     * </p>
     *
     * @param mimes the list of MIME types to be matched.
     *
     * @throws NullPointerException if <code>mimes</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>mimes</code> is empty; or contains instances other than strings; or
     *         contains empty strings.
     */
    public MimeMatcher(List mimes) {
        Validator.validateStringCollection(mimes, "mimes");

        this.mimes = new ArrayList(mimes);
    }

    /**
     * <p>
     * Creates a new instance of <code>MimeMatcher</code> class. The list of MIME types is read from the given
     * property in configuration.
     * </p>
     *
     * @param property the property where configurations are read from.
     *
     * @throws NullPointerException if <code>property</code> is <code>null</code>.
     * @throws ConfigurationException if <code>property</code> does not contain the required sub-property; or the
     *         sub-property does not contain any value; or the sub-property contains empty strings.
     */
    public MimeMatcher(Property property) throws ConfigurationException {
        Validator.validateNotNull(property, "property");

        String[] values = property.getValues(MIMES_PROPERTY_KEY);

        // Verify the property values
        Validator.validateStringArrayConfiguration(values, MIMES_PROPERTY_KEY);

        mimes = Arrays.asList(values);
    }

    /**
     * <p>
     * Gets a flag indicating whether the MIME type of the given file content type matches at least one of the MIME
     * types.
     * </p>
     *
     * @param type the file content type to be matched.
     *
     * @return <code>true</code> if the MIME type of the file content type matches at least one of the MIME types;
     *         <code>false</code> otherwise.
     */
    protected boolean match(FileType type) {
        return mimes.contains(type.getMime());
    }
}



