/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.type.FileType;

import com.topcoder.util.config.Property;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * <p>
 * Defines a concrete implementation of <code>FileMatcher</code> interface to provide the functionality to match a
 * file against a list of type names. The file type name should be the same as defined in Magic Numbers component.
 * The matching is done case-sensitively.
 * </p>
 *
 * <p>
 * The list of type names can be read from configuration under the property 'types'. This property must contain at
 * least one value. Empty strings are not allowed.
 * </p>
 *
 * <p>
 * This class is thead-safe, since it is immutable.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class TypeNameMatcher extends FileTypeMatcher {
    /** Represents the property name in configuration to read the list of type names. */
    public static final String TYPES_PROPERTY_KEY = "types";

    /** Represents the list of type names to be matched. */
    private List types;

    /**
     * <p>
     * Creates a new instance of <code>TypeNameMatcher</code> class. The list of type name contains only one given
     * type name.
     * </p>
     *
     * @param type the type name to be matched.
     *
     * @throws NullPointerException if <code>type</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>type</code> is empty string.
     */
    public TypeNameMatcher(String type) {
        Validator.validateNotNullOrEmpty(type, "type");

        types = new ArrayList();
        types.add(type);
    }

    /**
     * <p>
     * Creates a new instance of <code>TypeNameMatcher</code> class. The list of type names is given. The list should
     * only contain non-empty strings, and cannot be empty list. Later modifications to the given list does not
     * affect the internal data.
     * </p>
     *
     * @param types the list of type names to be matched.
     *
     * @throws NullPointerException if <code>types</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>types</code> is empty; or contains instances other than strings; or
     *         contains empty strings.
     */
    public TypeNameMatcher(List types) {
        Validator.validateStringCollection(types, "types");

        this.types = new ArrayList(types);
    }

    /**
     * <p>
     * Creates a new instance of <code>TypeNameMatcher</code> class. The list of type names is read from the given
     * property in configuration.
     * </p>
     *
     * @param property the property where configurations are read from.
     *
     * @throws NullPointerException if <code>property</code> is <code>null</code>.
     * @throws ConfigurationException if <code>property</code> does not contain the required sub-property; or the
     *         sub-property does not contain any value; or the sub-property contains empty strings.
     */
    public TypeNameMatcher(Property property) throws ConfigurationException {
        Validator.validateNotNull(property, "property");

        String[] values = property.getValues(TYPES_PROPERTY_KEY);

        // Verify the property values
        Validator.validateStringArrayConfiguration(values, TYPES_PROPERTY_KEY);

        types = Arrays.asList(values);
    }

    /**
     * <p>
     * Gets a flag indicating whether the name of the given file content type matches at least one of the type names.
     * </p>
     *
     * @param type the file content type to be matched.
     *
     * @return <code>true</code> if the name of the file content type matches at least one of the type names;
     *         <code>false</code> otherwise.
     */
    protected boolean match(FileType type) {
        return types.contains(type.getName());
    }
}



