/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.type.FileType;
import com.topcoder.file.type.SignatureChecker;
import com.topcoder.file.type.TextSignatureChecker;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.lang.reflect.Field;

import java.util.ArrayList;
import java.util.List;


/**
 * <p>
 * Tests functionality and error cases of <code>MimeMatcher</code> class. The <code>match(File)</code> method is
 * defined in its super class <code>FileTypeMatcher</code>. The accuracy and error cases of this method is already
 * tested in the test case of that class. Therefore, here only the protected <code>match</code> method is tested.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class MimeMatcherTestCase extends TestCase {
    /** Represents the namespace used in tests. */
    private static final String NAMESPACE = "test";

    /** Represents the property name used in tests. */
    private static final String PROPERTY = "test_property";

    /** Represents the <code>MimeMatcher</code> instance used in tests. */
    private MimeMatcher matcher;

    /** Represents the configuration manager instance used in tests. */
    private ConfigManager configManager;

    /**
     * <p>
     * Sets up the test environment. The test instance is created. The test namespace is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        configManager = ConfigManager.getInstance();

        // The matcher instance contains three MIME types
        List list = new ArrayList();

        list.add("image/x-png");
        list.add("application/x-rar");
        list.add("text/html");
        matcher = new MimeMatcher(list);

        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
    }

    /**
     * <p>
     * Cleans up the test environment. The test instance is disposed. The test namespace is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }

        matcher = null;
        configManager = null;
    }

    /**
     * <p>
     * Tests constructor <code>MimeMatcher(String)</code> when the given MIME type is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMimeMatcherStringNull() throws Exception {
        try {
            new MimeMatcher((String) null);
            fail("The MIME type is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>MimeMatcher(List)</code> when the given MIME list is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     */
    public void testMimeMatcherListNull() {
        try {
            new MimeMatcher((List) null);
            fail("The MIME list is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>MimeMatcher(List)</code> when the given MIME list is empty. Expect
     * IllegalArgumentException.
     * </p>
     */
    public void testMimeMatcherListEmpty() {
        try {
            new MimeMatcher(new ArrayList());
            fail("The MIME list is empty, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>MimeMatcher(List)</code> when the given MIME list contains empty string. Expect
     * IllegalArgumentException.
     * </p>
     */
    public void testMimeMatcherListEmptyString() {
        List list = new ArrayList();

        list.add("  ");

        try {
            new MimeMatcher(list);
            fail("The MIME list contains empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>MimeMatcher(List)</code> when the given MIME list contains non-string instance. Expect
     * IllegalArgumentException.
     * </p>
     */
    public void testMimeMatcherListNotString() {
        List list = new ArrayList();

        list.add(new Object());

        try {
            new MimeMatcher(list);
            fail("The MIME list contains non-string instance, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>MimeMatcher(Property)</code> when the given property is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMimeMatcherPropertyNull() throws Exception {
        try {
            new MimeMatcher((Property) null);
            fail("The property is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>MimeMatcher(Property)</code> when the MIME list is missing in configuration. Expect
     * ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMimeMatcherPropertyExtensionMissing()
        throws Exception {
        configManager.add("MimeMatcher/Missing.xml");

        try {
            new MimeMatcher(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The MIME list is missing in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>RegexMatcher(Property)</code> when the MIME type is empty string in configuration.
     * Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRegexMatcherPropertyPatternEmpty()
        throws Exception {
        configManager.add("MimeMatcher/EmptyString.xml");

        try {
            new RegexMatcher(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The MIME type is empty string in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>MimeMatcher(String)</code>. The internal MIME list should contain only one
     * MIME, which is given as the argument.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMimeMatcherStringAccuracy() throws Exception {
        matcher = new MimeMatcher("text/html");

        // Get the internal list
        Field field = MimeMatcher.class.getDeclaredField("mimes");
        List list;

        try {
            field.setAccessible(true);
            list = (List) field.get(matcher);
        } finally {
            field.setAccessible(false);
        }

        // Verify
        assertEquals("One MIME type should exist.", 1, list.size());
        assertEquals("The MIME type should be correct.", "text/html", list.get(0));
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>MimeMatcher(List)</code>. The internal MIME list should contain all MIME
     * types which are given as the argument. A copy of the given list should be made.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMimeMatcherListAccuracy() throws Exception {
        List list = new ArrayList();

        list.add("image/x-png");
        list.add("application/x-rar");
        list.add("text/html");

        matcher = new MimeMatcher(list);

        // Get the internal list
        Field field = MimeMatcher.class.getDeclaredField("mimes");
        List actual;

        try {
            field.setAccessible(true);
            actual = (List) field.get(matcher);
        } finally {
            field.setAccessible(false);
        }

        // Verify copy
        assertNotSame("A copy of the list should be made.", list, actual);

        // Verify the list
        assertEquals("The MIME list should be correct.", list, actual);
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>MimeMatcher(Property)</code>. The internal MIME list should contain all
     * MIME types which are specified in the configuration.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testMimeMatcherPropertyAccuracy() throws Exception {
        configManager.add("MimeMatcher/Valid.xml");

        matcher = new MimeMatcher(configManager.getPropertyObject(NAMESPACE, PROPERTY));

        // Get the internal list
        Field field = MimeMatcher.class.getDeclaredField("mimes");
        List actual;

        try {
            field.setAccessible(true);
            actual = (List) field.get(matcher);
        } finally {
            field.setAccessible(false);
        }

        // The list should be sorted.
        List expected = new ArrayList();

        expected.add("image/x-png");
        expected.add("application/x-rar");
        expected.add("text/html");

        // Verify the list
        assertEquals("The MIME list should be correct.", expected, actual);
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when the given MIME exists in the MIME list. The comparison should be
     * done case-sensitively.
     * </p>
     */
    public void testMatchStringExistAccuracy() {
        FileType type = new FileType("rar", new String[0], new String[0], "application/x-rar",
                new SignatureChecker[] {new TextSignatureChecker()});

        assertTrue("The MIME type exists in the MIME list.", matcher.match(type));
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when the given MIME does not exist in the MIME list. The comparison
     * should be done case-sensitively.
     * </p>
     */
    public void testMatchStringNotExistAccuracy() {
        FileType type = new FileType("RaR", new String[0], new String[0], "application/x-RaR",
                new SignatureChecker[] {new TextSignatureChecker()});

        assertFalse("The MIME type does not exist in the MIME list.", matcher.match(type));
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(MimeMatcherTestCase.class);
    }
}



