/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;


/**
 * <p>
 * Tests functionality and error cases of <code>AbstractStatsReporting</code> class. This class is an abstract class,
 * so its concrete implementation <code>BasicStatsReporting</code> is used for testing purposes, since the sub-class
 * delegates the super class' constructors, and does not override any methods in the super class. Property getters
 * are not tested individually, since the accuracy tests of such methods are combined into other accuracy test
 * cases.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class AbstractStatsReportingTestCase extends TestCase {
    /** Represents the <code>AbstractStatsReporting</code> instance used in tests. */
    private AbstractStatsReporting reporting;

    /** Represents the <code>FileProcessor</code> instance used in tests. */
    private FileProcessor processor;

    /** Represents the <code>StatsResult</code> instance used in tests. */
    private StatsResult result;

    /** Represents the <code>Metric</code> instance used in tests. */
    private Metric metric;

    /**
     * <p>
     * Sets up the test environment. The test instances are created.
     * </p>
     */
    protected void setUp() {
        reporting = new BasicStatsReporting();
        processor = new TextLineCounter("alias", new AnyFileMatcher());
        result = new StatsResult();
        metric = new Metric("Metric", "Metric", "Metric");
    }

    /**
     * <p>
     * Cleans up the test environment. The test instances are disposed.
     * </p>
     */
    protected void tearDown() {
        reporting = null;
        processor = null;
        result = null;
        metric = null;
    }

    /**
     * <p>
     * Tests constructor <code>AbstractStatsReporting(Property)</code> when the given property is <code>null</code>.
     * Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAbstractStatsReportingPropertyNull()
        throws Exception {
        try {
            new BasicStatsReporting(null);
            fail("The property is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>aggregate</code> when the file processor is <code>null</code>. Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAggregateProcessorNull() throws Exception {
        try {
            reporting.aggregate(null, new File("any"), result);
            fail("The file processor is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>aggregate</code> when the file is <code>null</code>. Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAggregateFileNull() throws Exception {
        try {
            reporting.aggregate(processor, null, result);
            fail("The file is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>aggregate</code> when the statistical result is <code>null</code>. Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAggregateResultNull() throws Exception {
        try {
            reporting.aggregate(processor, new File("any"), null);
            fail("The statistical result is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>aggregate</code> when the file is the base directory. Expect NullPointerException.
     * </p>
     */
    public void testAggregateFileIsBaseDir() {
        reporting.reset(new File("test_files"));

        try {
            reporting.aggregate(processor, new File("test_files"), result);
            fail("The file is the base directory, should throw StatsReporetingException.");
        } catch (StatsReportingException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>aggregate</code> when the file does not locate in the base directory. Expect NullPointerException.
     * </p>
     */
    public void testAggregateFileNotInBaseDir() {
        reporting.reset(new File("test_files"));

        try {
            reporting.aggregate(processor, new File("conf/notinbase"), result);
            fail("The file does not locate in the base directory, should throw StatsReporetingException.");
        } catch (StatsReportingException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>reset</code> when the base directory does not exist. Expect IllegalArgumentException.
     * </p>
     */
    public void testResetDirectoryNotExist() {
        try {
            reporting.reset(new File("notexist"));
            fail("The base directory does not exist, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>reset</code> when the base directory is actually a file. Expect IllegalArgumentException.
     * </p>
     */
    public void testResetDirectoryIsFile() {
        try {
            reporting.reset(new File("test_files/small.txt"));
            fail("The base directory is a file, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>aggregate</code> when the base directory is <code>null</code>. The file path should be
     * treated as the absolute path. The results of each parent directory of the given file should be aggregated.
     * The results of the given processor should be aggregated. The results of the file should be aggregated. The
     * overall result should be aggregated.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAggregateNoBaseDirectoryAccuracy()
        throws Exception {
        File file = new File("test_files/FileProcessorBase/Valid.xml");

        result.setStat(metric, 10);

        // Aggregate a result
        reporting.aggregate(processor, file, result);

        result.setStat(metric, 100);

        // Aggregate a different result
        reporting.aggregate(processor, file, result);

        // Verify the aggregated results
        assertEquals("The processor result should be correct.", 110,
            reporting.getAliasStats().get("alias").getStat(metric));
        assertEquals("The file result should be correct.", 110,
            reporting.getFileStats().get(file.getPath()).getStat(metric));
        assertEquals("The directory result should be correct.", 110,
            reporting.getDirectoryStats().get(new File("test_files", "FileProcessorBase").getPath()).getStat(metric));
        assertEquals("The directory result should be correct.", 110,
            reporting.getDirectoryStats().get("test_files").getStat(metric));
        assertEquals("The overall result should be correct.", 110, reporting.getOverallStats().getStat(metric));
    }

    /**
     * <p>
     * Tests accuracy of <code>aggregate</code> when the base directory is specified. The file path should be
     * modified as a relative path to the base directory. The results of each parent directory of the given file
     * should be aggregated. The results of the given processor should be aggregated. The results of the file should
     * be aggregated. The overall result should be aggregated.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testAggregateBaseDirectoryAccuracy() throws Exception {
        File file = new File("test_files/FileProcessorBase/Valid.xml");

        // Set a base directory
        reporting.reset(new File("test_files"));

        result.setStat(metric, 10);

        // Aggregate a result
        reporting.aggregate(processor, file, result);

        result.setStat(metric, 100);

        // Aggregate a different result
        reporting.aggregate(processor, file, result);

        // Verify the aggregated results
        assertEquals("The processor result should be correct.", 110,
            reporting.getAliasStats().get("alias").getStat(metric));
        assertEquals("The file result should be correct.", 110,
            reporting.getFileStats().get(new File("FileProcessorBase", "Valid.xml").getPath()).getStat(metric));
        assertEquals("The directory result should be correct.", 110,
            reporting.getDirectoryStats().get("FileProcessorBase").getStat(metric));
        assertEquals("The overall result should be correct.", 110, reporting.getOverallStats().getStat(metric));
    }

    /**
     * <p>
     * Tests accuracy of <code>reset</code> when the given base directory is <code>null</code>. The current
     * statistical results should be cleared. The base directory should be set to <code>null</code>, which means
     * unspecified.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testResetBaseDirNullAccuracy() throws Exception {
        // First, aggregate some results
        File file = new File("test_files/FileProcessorBase/Valid.xml");

        result.setStat(metric, 10);
        reporting.aggregate(processor, file, result);

        // Reset
        reporting.reset(null);

        // Verify the base directory
        assertNull("The base directory should be set to unspecified.", reporting.getBaseDirectory());

        // Verify empty aggregated results
        assertTrue("The overall result should be cleared.", reporting.getOverallStats().getMetrics().isEmpty());
        assertTrue("The directory results should be cleared.", reporting.getDirectoryStats().getAllKeys().isEmpty());
        assertTrue("The file results should be cleared.", reporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("The processor results should be cleared.", reporting.getAliasStats().getAllKeys().isEmpty());
    }

    /**
     * <p>
     * Tests accuracy of <code>reset</code> when the given base directory is an existing directory. The current
     * statistical results should be cleared. The base directory should be set to the given directory.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testResetBaseDirAccuracy() throws Exception {
        // First, aggregate some results
        File file = new File("test_files/FileProcessorBase/Valid.xml");

        result.setStat(metric, 10);
        reporting.aggregate(processor, file, result);

        // Reset
        reporting.reset(new File("test_files"));

        // Verify the base directory
        assertEquals("The base directory should be set to the given directory.", new File("test_files"),
            reporting.getBaseDirectory());

        // Verify empty aggregated results
        assertTrue("The overall result should be cleared.", reporting.getOverallStats().getMetrics().isEmpty());
        assertTrue("The directory results should be cleared.", reporting.getDirectoryStats().getAllKeys().isEmpty());
        assertTrue("The file results should be cleared.", reporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("The processor results should be cleared.", reporting.getAliasStats().getAllKeys().isEmpty());
    }

    /**
     * <p>
     * Tests accuracy of <code>getOverallStats</code>. A copy of the internal overall statistical result should be
     * returned. Thus, each time, a different instance should be returned.
     * </p>
     */
    public void testGetOverallStatsCopyAccuracy() {
        StatsResult overall = reporting.getOverallStats();

        assertNotSame("A copy of overall statistical result should be returned.", overall, reporting.getOverallStats());
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(AbstractStatsReportingTestCase.class);
    }
}



