/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;

import com.topcoder.util.config.Property;

import java.io.PrintWriter;
import java.io.StringWriter;

import java.util.Iterator;
import java.util.List;


/**
 * <p>
 * Defines a concrete implementation of <code>StatsReporting</code> interface to provide the functionality to render
 * human-readable reports. The rendered report should include all information aggregated by the
 * <code>AbstractStatsReporting</code> class. The information includes the overall statistical result and results
 * for each file, directory and processor.
 * </p>
 *
 * <p>
 * This class requires no configuration.
 * </p>
 *
 * <p>
 * This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class BasicStatsReporting extends AbstractStatsReporting {
    /** Represents the header of the statistics report. */
    private static final String HEADER = "File Statistics Report";

    /** Represents a double dashed line. */
    private static final String SEPARATOR_DOUBLE_LINE = "========================================";

    /** Represents a single dashed line. */
    private static final String SEPARATOR_SINGLE_LINE = "----------------------------------------";

    /** Represents the header of the statistics of files. */
    private static final String FILES_HEADER = "Statistics of files:";

    /** Represents the header of the statistics of directories. */
    private static final String DIRECTORIES_HEADER = "Statistics of directories:";

    /** Represents the header of the statistics of processors. */
    private static final String PROCESSORS_HEADER = "Statistics of processors:";

    /** Represents the header of overall statistics. */
    private static final String OVERALL_HEADER = "Overall statistics:";

    /**
     * <p>
     * Creates a new instance of <code>BasicStatsReporting</code> class.
     * </p>
     */
    public BasicStatsReporting() {
    }

    /**
     * <p>
     * Creates a new instance of <code>BasicStatsReporting</code> class. This class requires no configuration.
     * </p>
     *
     * @param property the property where configurations are read from.
     *
     * @throws NullPointerException if <code>property</code> is <code>null</code>.
     * @throws ConfigurationException if error occurs when loading configuration; or mandatory property is missing;
     *         or invalid property value exists.
     */
    public BasicStatsReporting(Property property) throws ConfigurationException {
        super(property);
    }

    /**
     * <p>
     * Generates a human-readable statistics report according to the current aggregated results. The information
     * includes the overall statistical result and results for each file, directory and processor.
     * </p>
     *
     * @return a human-readable statistics report of the current aggregated results.
     */
    public String generateReport() {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);

        // Render the header
        pw.println(HEADER);
        pw.println(SEPARATOR_DOUBLE_LINE);

        // Create the files statistics. The file names are rendered in a case-insensitive order.
        renderGroup(getFileStats(), FILES_HEADER, pw);

        // Create the directories statistics. The directory names are rendered in a case-insensitive order.
        renderGroup(getDirectoryStats(), DIRECTORIES_HEADER, pw);

        // Create the processors statistics. The processor aliases are rendered in a case-insensitive order.
        renderGroup(getAliasStats(), PROCESSORS_HEADER, pw);

        // Render the overall statistics
        pw.println(OVERALL_HEADER);
        renderStatsResult(getOverallStats(), "\t", pw);
        pw.println(SEPARATOR_DOUBLE_LINE);

        pw.close();

        return sw.toString();
    }

    /**
     * <p>
     * Renders a statistics collection to the given print writer. The whole collection is rendered under a group
     * header. Each item is separated by a single dashed line. A double dashed line is followed by the rendered
     * collection.
     * </p>
     *
     * @param stats the statistics collection to be rendered.
     * @param groupHeader the header of the statistics collection in the report.
     * @param pw the print writer where the report is written to.
     */
    private void renderGroup(StatsCollection stats, String groupHeader, PrintWriter pw) {
        // Render the group header
        pw.println(groupHeader);

        // Get the sorted keys
        List keys = stats.getAllKeys();

        // Enumerate every sorted key in result statistics
        Iterator iter = keys.iterator();

        while (iter.hasNext()) {
            // Get the statistics for this item
            String key = (String) iter.next();
            StatsResult stat = stats.get(key);

            // Render the header of item.
            pw.println(SEPARATOR_SINGLE_LINE);
            pw.println("\t" + key + ":");

            // Render the result.
            renderStatsResult(stat, "\t\t", pw);
        }

        // Render the end of a group
        pw.println(SEPARATOR_DOUBLE_LINE);
    }

    /**
     * <p>
     * Renders metrics in a statistics result to the given print writer. For each metric, an indent is rendered
     * before the actual data.
     * </p>
     *
     * @param stat the statistics result to be rendered.
     * @param indent the indent of each metric.
     * @param pw the print writer where the report is written to.
     */
    private void renderStatsResult(StatsResult stat, String indent, PrintWriter pw) {
        // Render every metric
        Iterator metricIter = stat.getMetrics().iterator();

        while (metricIter.hasNext()) {
            // Get the metric
            Metric metric = (Metric) metricIter.next();
            String displayName = metric.getDisplayName();

            // Render the metric and the statistic
            pw.println(indent + displayName + ":" + stat.getStat(metric));
        }
    }
}




