/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.BinaryFileMatcher;
import com.topcoder.file.statistics.matcher.MockFileNameMatcher;
import com.topcoder.file.statistics.matcher.TextFileMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.BasicStatsReporting;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;

import com.topcoder.util.config.ConfigManager;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;
import java.io.IOException;

import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * <p>
 * Tests functionality and error cases of <code>FileStatistics</code> class. Accuracy tests of some property getters
 * are tested together with the accuracy tests of property setters, since they cannot be tested individually.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class FileStatisticsTestCase extends TestCase {
    /** Represents the namespace used in tests. */
    private static final String NAMESPACE = "test";

    /** Represents the <code>FileStatistics</code> instance used in tests. */
    private FileStatistics fileStat;

    /** Represents the configuration manager instance used in tests. */
    private ConfigManager configManager;

    /** Represents the <code>FileProcessor</code> instance used in tests. */
    private FileProcessor processor;

    /**
     * <p>
     * Sets up the test environment. The test instances are created. The test configuration is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        fileStat = new FileStatistics();
        processor = new TextLineCounter("processor1", new AnyFileMatcher());
        configManager = ConfigManager.getInstance();

        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
    }

    /**
     * <p>
     * Cleans up the test environment. The test instances are disposed. The test configuration is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }

        configManager = null;
        fileStat = null;
        processor = null;
    }

    /**
     * <p>
     * Tests constructor <code>FileStatistics(String)</code> when the given namespace is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileStatisticsStringNull() throws Exception {
        try {
            new FileStatistics(null);
            fail("The namespace is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileStatistics(String)</code> when the given namespace is empty string. Expect
     * IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileStatisticsStringEmpty() throws Exception {
        try {
            new FileStatistics("  ");
            fail("The namespace is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileStatistics(String)</code> when the namespace does not exist in configuration.
     * Expect ConfigurationException.
     * </p>
     */
    public void testFileStatisticsStringNamespaceUnknown() {
        try {
            new FileStatistics("Unknown");
            fail("The namespace is unknown, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileStatistics(String)</code> when the property 'processors' does not exist in
     * configuration. Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileStatisticsStringProcessorsMissing()
        throws Exception {
        configManager.add("FileStatistics/ProcessorsMissing.xml");

        try {
            new FileStatistics(NAMESPACE);
            fail("The property 'processors' is missing in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileStatistics(String)</code> when the class name of a processor does not exist in
     * configuration. Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileStatisticsStringProcessorClassMissing()
        throws Exception {
        configManager.add("FileStatistics/ProcessorClassMissing.xml");

        try {
            new FileStatistics(NAMESPACE);
            fail("The class name of a processor is missing in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileStatistics(String)</code> when the instance of a processor cannot be created.
     * Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileStatisticsStringProcessorClassError()
        throws Exception {
        configManager.add("FileStatistics/ProcessorClassError.xml");

        try {
            new FileStatistics(NAMESPACE);
            fail("The instance of a processor cannot be created, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileStatistics(String)</code> when the class name of the reporting does not exist in
     * configuration. Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileStatisticsStringReportClassMissing()
        throws Exception {
        configManager.add("FileStatistics/ReportClassMissing.xml");

        try {
            new FileStatistics(NAMESPACE);
            fail("The class name of the reporting is missing in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>FileStatistics(String)</code> when the instance of the reporting cannot be created.
     * Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileStatisticsStringReportClassError()
        throws Exception {
        configManager.add("FileStatistics/ReportClassError.xml");

        try {
            new FileStatistics(NAMESPACE);
            fail("The instance of the reporting cannot be created, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>addFileProcessor</code> when the given file processor is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     */
    public void testAddFileProcessorNull() {
        try {
            fileStat.addFileProcessor(null);
            fail("The file processor is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>removeFileProcessor</code> when the given alias is <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testRemoveFileProcessorNull() {
        try {
            fileStat.removeFileProcessor(null);
            fail("The alias is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>removeFileProcessor</code> when the given alias is empty string. Expect IllegalArgumentException.
     * </p>
     */
    public void testRemoveFileProcessorEmpty() {
        try {
            fileStat.removeFileProcessor("  ");
            fail("The alias is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>getFileProcessor</code> when the given alias is <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testGetFileProcessorNull() {
        try {
            fileStat.getFileProcessor(null);
            fail("The alias is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>getFileProcessor</code> when the given alias is empty string. Expect IllegalArgumentException.
     * </p>
     */
    public void testGetFileProcessorEmpty() {
        try {
            fileStat.getFileProcessor("  ");
            fail("The alias is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>setReporting</code> when the given reporting is <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testSetReportingNull() {
        try {
            fileStat.setReporting(null);
            fail("The reporting is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>processFile</code> when the file to be processed is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessFileNull() throws Exception {
        try {
            fileStat.processFile(null);
            fail("The file to be processed is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>processFile</code> when I/O error occurs during the process of file. Expect IOException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessFileIOError() throws Exception {
        fileStat.addFileProcessor(processor);

        try {
            fileStat.processFile(new File("notexist"));
            fail("I/O error occurs during process of file, should throw IOException.");
        } catch (IOException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>processFile</code> when error except I/O error occurs during the process of file. Expect
     * FileStatisticsException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessFileProcessingError() throws Exception {
        fileStat.addFileProcessor(new TextLineCounter("error", new MockFileNameMatcher()));

        try {
            fileStat.processFile(new File("test_files/error"));
            fail("An error except I/O error occurs during the process of file, should throw FileStatisticsException.");
        } catch (FileStatisticsException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>FileStatistics()</code>. The list of file processors should be empty. The matching all
     * flag should be <code>false</code>. The reporting should be an instance of <code>XmlStatsReporting</code>
     * class.
     * </p>
     */
    public void testFileStatisticsAccuracy() {
        fileStat = new FileStatistics();

        // Verify properties
        assertTrue("The list of file processors should be empty.", fileStat.getAllFileProcessors().isEmpty());
        assertFalse("The matching all flag should be false.", fileStat.isMatchAll());
        assertTrue("The reporting should be an instance of XmlStatsReporting.",
            fileStat.getReporting() instanceof XmlStatsReporting);
    }

    /**
     * <p>
     * Tests accuracy of <code>FileStatistics(String)</code> when all optional properties are specified. According to
     * the configuration file, the list of file processors should contain two file processors. The first one is
     * <code>TextLineCounter</code>. The second one is <code>CStyleLineCounter</code>. The reporting should be an
     * instance of <code>BasicStatsReporting</code> class. The matching all flag should be <code>true</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileStatisticsStringAllSpecifiedAccuracy()
        throws Exception {
        configManager.add("FileStatistics/Valid.xml");

        fileStat = new FileStatistics(NAMESPACE);

        // Verify properties
        List processors = fileStat.getAllFileProcessors();

        assertEquals("The list of file processors should contain 2 processors.", 2, processors.size());
        assertTrue("The file processor should be TextLineCounter.", processors.get(0) instanceof TextLineCounter);
        assertTrue("The file processor should be CStyleLineCounter.", processors.get(1) instanceof CStyleLineCounter);
        assertTrue("The matching all flag should be true.", fileStat.isMatchAll());
        assertTrue("The reporting should be an instance of BasicStatsReporting.",
            fileStat.getReporting() instanceof BasicStatsReporting);
    }

    /**
     * <p>
     * Tests accuracy of <code>FileStatistics(String)</code> when the reporting is not specified. According to the
     * configuration file, the list of file processors should contain one file processor with the type
     * <code>TextLineCounter</code>. The reporting should be an instance of <code>XmlStatsReporting</code> class by
     * default. The matching all flag should be <code>true</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileStatisticsStringReportMissingAccuracy()
        throws Exception {
        configManager.add("FileStatistics/ReportMissing.xml");

        fileStat = new FileStatistics(NAMESPACE);

        // Verify properties
        List processors = fileStat.getAllFileProcessors();

        assertEquals("The list of file processors should contain 1 processor.", 1, processors.size());
        assertTrue("The file processor should be TextLineCounter.", processors.get(0) instanceof TextLineCounter);
        assertTrue("The matching all flag should be true.", fileStat.isMatchAll());
        assertTrue("The reporting should be an instance of XmlStatsReporting.",
            fileStat.getReporting() instanceof XmlStatsReporting);
    }

    /**
     * <p>
     * Tests accuracy of <code>FileStatistics(String)</code> when the matching all flag is not specified. According
     * to the configuration file, the list of file processors should contain one file processor with the type
     * <code>TextLineCounter</code>. The reporting should be an instance of <code>BasicStatsReporting</code> class.
     * The matching all flag should be <code>false</code> by default.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testFileStatisticsStringMatchAllMissingAccuracy()
        throws Exception {
        configManager.add("FileStatistics/MatchAllMissing.xml");

        fileStat = new FileStatistics(NAMESPACE);

        // Verify properties
        List processors = fileStat.getAllFileProcessors();

        assertEquals("The list of file processors should contain 1 processor.", 1, processors.size());
        assertTrue("The file processor should be TextLineCounter.", processors.get(0) instanceof TextLineCounter);
        assertFalse("The matching all flag should be false.", fileStat.isMatchAll());
        assertTrue("The reporting should be an instance of BasicStatsReporting.",
            fileStat.getReporting() instanceof BasicStatsReporting);
    }

    /**
     * <p>
     * Tests accuracy of <code>addFileProcessor</code> when the alias of the given file processor does not exist. The
     * file processor should be appended to the end of the file processor list.
     * </p>
     */
    public void testAddFileProcessorNotExistAccuracy() {
        FileProcessor processor2 = new CStyleLineCounter("processor2", new AnyFileMatcher());

        // Add two processors
        fileStat.addFileProcessor(processor);
        fileStat.addFileProcessor(processor2);

        // Verify the existence
        assertEquals("The first processor should be added.", processor, fileStat.getFileProcessor("processor1"));
        assertEquals("The second processor should be added.", processor2, fileStat.getFileProcessor("processor2"));

        // Verify the order of processors
        List processors = fileStat.getAllFileProcessors();

        assertEquals("The first processor should be the first.", processor, processors.get(0));
        assertEquals("The second processor should be the second.", processor2, processors.get(1));
    }

    /**
     * <p>
     * Tests accuracy of <code>addFileProcessor</code> when the alias of the given file processor already exists. The
     * original file processor should be removed, and the new file processor should be appended to the end of the
     * file processor list.
     * </p>
     */
    public void testAddFileProcessorExistAccuracy() {
        FileProcessor processor2 = new CStyleLineCounter("processor2", new AnyFileMatcher());

        // Add two processors
        fileStat.addFileProcessor(processor);
        fileStat.addFileProcessor(processor2);

        // Add a new processor with the alias of the first processor
        processor = new TextLineCounter("processor1", new TextFileMatcher());
        fileStat.addFileProcessor(processor);

        // Verify the existence
        assertEquals("The new processor should replace the previous one.", processor,
            fileStat.getFileProcessor("processor1"));

        // Verify the order of processors
        List processors = fileStat.getAllFileProcessors();

        assertEquals("The original processor should be removed.", processor2, processors.get(0));
        assertEquals("The new processor should be appended.", processor, processors.get(1));
    }

    /**
     * <p>
     * Tests accuracy of <code>removeFileProcessor</code> when the given alias exists. The file processor should be
     * removed and returned.
     * </p>
     */
    public void testRemoveFileProcessorExistAccuracy() {
        fileStat.addFileProcessor(processor);

        // Remove the processor and verify
        assertEquals("The removed file processor should be returned.", processor,
            fileStat.removeFileProcessor("processor1"));

        // Verify the removal of the processor. Both map and list are verified.
        assertFalse("The file processor should be removed.", fileStat.getAllFileProcessors().contains(processor));
        assertNull("The file processor should be removed.", fileStat.getFileProcessor("processor1"));
    }

    /**
     * <p>
     * Tests accuracy of <code>removeFileProcessor</code> when the given alias does not exist. It should return
     * <code>null</code>.
     * </p>
     */
    public void testRemoveFileProcessorNotExistAccuracy() {
        assertNull("When the alias does not exist, null should be returned", fileStat.removeFileProcessor("notexist"));
    }

    /**
     * <p>
     * Tests accuracy of <code>getFileProcessor</code> when the given alias exists. The correct file processor should
     * be returned.
     * </p>
     */
    public void testGetFileProcessorExistAccuracy() {
        fileStat.addFileProcessor(processor);

        // Get the processor and verify
        assertEquals("The correct file processor should be returned.", processor,
            fileStat.getFileProcessor("processor1"));
    }

    /**
     * <p>
     * Tests accuracy of <code>getFileProcessor</code> when the given alias does not exist. It should return
     * <code>null</code>.
     * </p>
     */
    public void testGetFileProcessorNotExistAccuracy() {
        assertNull("When the alias does not exist, null should be returned", fileStat.getFileProcessor("notexist"));
    }

    /**
     * <p>
     * Tests accuracy of <code>getAllFileProcessors</code>. A copy of the list of file processors should be returned.
     * </p>
     */
    public void testGetAllFileProcessorsCopyAccuracy() {
        List list = fileStat.getAllFileProcessors();

        // Verify a copy
        assertNotSame("A copy should be returned.", list, fileStat.getAllFileProcessors());
    }

    /**
     * <p>
     * Tests accuracy of <code>clearFileProcessors</code>. Both the list of file processors and the map of aliases
     * and file processors should be cleared.
     * </p>
     */
    public void testClearFileProcessorsAccuracy() {
        fileStat.addFileProcessor(processor);

        // Clear
        fileStat.clearAllFileProcessors();

        // Verify map and list
        assertTrue("The list of file processors should be empty.", fileStat.getAllFileProcessors().isEmpty());
        assertNull("The map of aliases and file processors should be empty.", fileStat.getFileProcessor("processor1"));
    }

    /**
     * <p>
     * Tests accuracy of <code>setReporting</code>. The reporting should be correctly set and can be retrieved by
     * <code>getReporting</code>. This method also tests accuracy of <code>getReporting</code>.
     * </p>
     */
    public void testSetReportingAccuracy() {
        StatsReporting reporting = new BasicStatsReporting();

        fileStat.setReporting(reporting);

        // Verify
        assertEquals("The reporting instance should be set and retrieved correctly.", reporting,
            fileStat.getReporting());
    }

    /**
     * <p>
     * Tests accuracy of <code>setMatchAll</code>. The matching all flag should be correctly set and can be retrieved
     * by <code>isMatchAll</code>. This method also tests accuracy of <code>isMatchAll</code>.
     * </p>
     */
    public void testSetMatchAllAccuracy() {
        fileStat.setMatchAll(true);

        // Verify
        assertTrue("The matching all flag should be true as set.", fileStat.isMatchAll());

        fileStat.setMatchAll(false);

        // Verify
        assertFalse("The matching all flag should be false as set.", fileStat.isMatchAll());
    }

    /**
     * <p>
     * Tests accuracy of <code>processFile</code> when matching all flag is <code>true</code>. All file processors
     * whose file matcher matches the file should be used to collect statistical data. Three processors are in the
     * list. One does not match the file, and the other two matches the file. The result should be the result of two
     * file processors.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessFileMatchAllAccuracy() throws Exception {
        // Add one processor never matches a text file
        fileStat.addFileProcessor(new CStyleLineCounter("never match", new BinaryFileMatcher()));

        // Add one text line counter matches any file
        fileStat.addFileProcessor(new TextLineCounter("text", new AnyFileMatcher()));

        // Add one C-style line counter matches any file
        fileStat.addFileProcessor(new CStyleLineCounter("c", new AnyFileMatcher()));

        // Set match all flag
        fileStat.setMatchAll(true);

        // Process a text file
        fileStat.processFile(new File("test_files/strange.c"));

        // Get and verify the report
        String report = fileStat.getReporting().generateReport();

        TestHelper.assertEquals("The report should be correct.", new File("test_files/MatchAll.xml"), report);
    }

    /**
     * <p>
     * Tests accuracy of <code>processFile</code> when matching all flag is <code>false</code>. Only the fist file
     * processor whose file matcher matches the file should be used to collect statistical data. Three processors
     * are in the list. One does not match the file, and the other two matches the file. The result should be the
     * result of the second file processor.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessFileMatchFirstAccuracy() throws Exception {
        // Add one processor never matches a text file
        fileStat.addFileProcessor(new CStyleLineCounter("never match", new BinaryFileMatcher()));

        // Add one text line counter matches any file, this is the only one should be used.
        fileStat.addFileProcessor(new TextLineCounter("text", new AnyFileMatcher()));

        // Add one C-style line counter matches any file
        fileStat.addFileProcessor(new CStyleLineCounter("c", new AnyFileMatcher()));

        // Process a text file
        fileStat.processFile(new File("test_files/strange.c"));

        // Get and verify the report
        String report = fileStat.getReporting().generateReport();

        TestHelper.assertEquals("The report should be correct.", new File("test_files/MatchFirst.xml"), report);
    }

    /**
     * <p>
     * Tests accuracy of <code>getArgumentHandlers</code> when there is no argument handler available. The returned
     * list should be an empty list.
     * </p>
     *
     * @since 2.0b
     */
    public void testGetArgumentHandlersNoHandlerAccuracy() {
        fileStat = new FileStatistics();
        fileStat.setReporting(new BasicStatsReporting());

        List list = fileStat.getArgumentHandlers();

        assertTrue("The list should be empty when no handler available.", list.isEmpty());
    }

    /**
     * <p>
     * Tests accuracy of <code>getArgumentHandlers</code> when there are argument handlers. The returned list should
     * contain a file processor, a file matcher, a reporting and the file statistics itself.
     * </p>
     *
     * @since 2.0b
     */
    public void testGetArgumentHandlersAccuracy() {
        fileStat = new MockFileStatistics();

        fileStat.setReporting(new XmlStatsReporting());
        fileStat.addFileProcessor(new TextLineCounter("processor", new MockFileMatcher()));

        // Get the list
        List list = fileStat.getArgumentHandlers();

        // Build the expected set
        Set expected = new HashSet();

        expected.add(fileStat);
        expected.add(fileStat.getReporting());
        expected.add(fileStat.getFileProcessor("processor"));
        expected.add(fileStat.getFileProcessor("processor").getFileMatcher());

        assertEquals("The list should contain a file processor, a file matcher, a reporting and the instance itself.",
            expected, new HashSet(list));
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(FileStatisticsTestCase.class);
    }
}



