/*
 * $(#)RegexMatcherTests.java        Sep 6, 2005
 *
 * Copyright (c) 2005, TopCoder Inc. All rights reserved.
 */

package com.topcoder.file.statistics.failuretests.matcher;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import junit.framework.TestCase;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.matcher.RegexMatcher;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;

/**
 * This test case contains the failure tests for the <code>RegexMatcher</code>
 * class.
 *
 * @author MPhk
 * @version 1.0
 */
public class RegexMatcherTests extends TestCase {

    /**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.matcher.failuretests";

    /**
     * The config file for this test.
     */
    private static final String CONFIG_FILE = "failuretests/matcher/config.xml";

    /**
     * An RegexMatcher instance to test.
     */
    private RegexMatcher matcher = null;

    /**
     * Load the configuration before testing.
     *
     * @throws Exception in case anything goes wrong
     */
    protected void setUp() throws Exception {
        // clear the namespace first
        ConfigManager configManager = ConfigManager.getInstance();
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
        // load the configuration
        ConfigManager.getInstance().add(CONFIG_FILE);

        matcher = new RegexMatcher(Pattern.compile("^.*\\.txt$"));
    }

    /**
     * Remove the configuration after testing.
     *
     * @throws Exception in case anything goes wrong
     */
    protected void tearDown() throws Exception {
        ConfigManager configManager = ConfigManager.getInstance();
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
    }

    /**
     * Checks whether the constructor RegexMatcher(Pattern) properly throws
     * NullPointerException for null argument.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testRegexMatcher1() throws Exception {
        try {
            new RegexMatcher((Pattern)null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor RegexMatcher(Property) properly throws
     * NullPointerException for null argument.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testRegexMatcher2() throws Exception {
        try {
            new RegexMatcher((Property)null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor RegexMatcher(Property) properly throws
     * Configuration for invalid property - no values in "extensions" sub-property.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testRegexMatcher3() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "regex_invalid1");

        try {
            new RegexMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor RegexMatcher(Property) properly throws
     * Configuration for invalid property - no "extensions" sub-property.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testRegexMatcher4() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "regex_invalid2");

        try {
            new RegexMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor RegexMatcher(Property) properly throws
     * Configuration for invalid property - wrong pattern.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testRegexMatcher5() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "regex_invalid3");

        try {
            new RegexMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor RegexMatcher(List) properly throws
     * NullPointerException for null argument.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testRegexMatcher6() throws Exception {
        try {
            new RegexMatcher((List)null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor RegexMatcher(List) properly throws
     * IllegalArgumentException for empty List argument.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testRegexMatcher7() throws Exception {
        try {
            new RegexMatcher(new ArrayList());
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor RegexMatcher(List) properly throws
     * IllegalArgumentException for List argument containing nulls.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testRegexMatcher8() throws Exception {
        List list = new ArrayList();
        list.add(Pattern.compile("^.*\\.txt$"));
        list.add(null);
        try {
            new RegexMatcher(list);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * Checks whether the constructor RegexMatcher(List) properly throws
     * IllegalArgumentException for List argument containing non String elements.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testRegexMatcher9() throws Exception {
        List list = new ArrayList();
        list.add(Pattern.compile("^.*\\.txt$"));
        list.add(new Object());
        try {
            new RegexMatcher(list);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * Tests the match method with null argument. NullPointerException should be thrown.
     *
     * @throws Exception in case anything goes wrong
     */
    public void testMatch() throws Exception {
        try {
            matcher.match((File) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

}
